#ifndef ZRE_VM_H
#define ZRE_VM_H

#include "common.h"

typedef union {
  float f;
  uint32_t u;
} Data;

typedef enum {
  OP_HALT,          /* terminate execution */
  OP_ADD,           /* dest = src1 + src2  */
  OP_SUB,           /* dest = src1 - src2  */
  OP_MUL,           /* dest = src1 * src2  */
  OP_DIV,           /* dest = src1 / src2  */
  OP_ADD_F32,       /* dest = src1 + src2  */
  OP_SUB_F32,       /* dest = src1 - src2  */
  OP_MUL_F32,       /* dest = src1 * src2  */
  OP_DIV_F32,       /* dest = src1 / src2  */
  OP_F32_TO_INT,    /* dest = src1 as int  */
  OP_INT_TO_F32,    /* dest = src1 as f32  */
  OP_MOV,           /* dest = src1	   */
  OP_JMP,           /* jump to address src1 unconditionally */
  OP_JGZ,           /* jump to address dest if src1 > 0 */
  OP_INT_TO_STRING, /* dest = src1 as str */
  OP_F32_TO_STRING, /* dest = src2 as str */
  OP_READ_STRING,
  OP_PRINT_STRING,
  OP_CMP_STRING,
} Opcode;

void run_vm(Data *memory, uint32_t memory_size);
int core_dump(Data *memory, uint32_t memory_size);

#endif
